# Package:  udns
# Date:     30th August 2014
# Author:   James Mills, prologic at shortcircuit dot net dot au


"""udns - a micro DNS Server"""


from warnings import filterwarnings


filterwarnings("ignore", "redis-py works best with hiredis.*")


__version__ = "0.0.1"
